-------------------------------------------------------------------------------
--|> INIT
--: Loads saved options or applies defaults.
--: Checks for database changes and applies updates when needed.
--: Loads player data.
-------------------------------------------------------------------------------
_G.weizPVP = LibStub("AceAddon-3.0"):NewAddon("weizPVP", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local weizPVP = weizPVP

-------------------------------------------------------------------------------
--|> ADDON INFO
-------------------------------------------------------------------------------
--> Addon Constants
weizPVP.Addon = {
    VERSION = "1.73.1",
    VERSION_NUM = 1731,
    DB_VERSION = "3.0",
    SPLASH_SCREEN_FOR_VERSIONS_UNDER = 1730 --> If upgrading from a version less than this, the splash screen will display
}

-------------------------------------------------------------------------------
--|> KEY BINDINGS
-------------------------------------------------------------------------------
--> Bindings Globals
BINDING_HEADER_WEIZPVP = "weizPVP"
BINDING_NAME_WEIZPVP_TOGGLECROSSHAIRS = "Toggle Crosshairs"

-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local C_UI_Reload = C_UI.Reload
local C_Timer_After = C_Timer.After
local UnitLevel = UnitLevel
local GetRealmName = GetRealmName
local GetUnitName = GetUnitName
local MAX_PLAYER_LEVEL_TABLE = MAX_PLAYER_LEVEL_TABLE
local GetExpansionLevel = GetExpansionLevel
local UnitFactionGroup = UnitFactionGroup
local select = select
local UIParent = UIParent
local GetScreenWidth = GetScreenWidth
local GetScreenHeight = GetScreenHeight
local GetFramerate = GetFramerate
local GetCVar = GetCVar
local wipe = wipe

-------------------------------------------------------------------------------
--|> GET CURRENT PLAYER AND GAME INFO
-------------------------------------------------------------------------------
--> Player Info
weizPVP.Player = {}
weizPVP.Player.Level = UnitLevel("player")
weizPVP.Player.CurrentRealm = GetRealmName()
weizPVP.Player.Name = GetUnitName("player")
weizPVP.Player.Faction = select(1, UnitFactionGroup("player"))
--> Game Info
weizPVP.Game = {}
weizPVP.Game.MaxPlayerLevel = MAX_PLAYER_LEVEL_TABLE[GetExpansionLevel()]

-------------------------------------------------------------------------------
--|> GET DISPLAY INFO
-------------------------------------------------------------------------------
--> Get Display Info
weizPVP.Display = {}
weizPVP.Display.Width = GetScreenWidth()
weizPVP.Display.Height = GetScreenHeight()
weizPVP.Display.xFactor = (768 / weizPVP.Display.Width) * (weizPVP.Display.Width / weizPVP.Display.Height)
weizPVP.Display.yFactor = 768 / weizPVP.Display.Height
weizPVP.Display.UIScale = UIParent:GetScale()
weizPVP.Display.EffectiveScale = UIParent:GetEffectiveScale()
weizPVP.Display.CVarUiScale = GetCVar("uiScale")
weizPVP.Display.CurrentFPS = GetFramerate()

-------------------------------------------------------------------------------
--|> SHARED MEDIA ASSETS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

--> Status Bars
SM:Register("STATUSBAR", "weizPVP: StatusBar", [[Interface\Addons\weizPVP\Media\Textures\bar-solid75.tga]])
SM:Register("STATUSBAR", "weizPVP: SolidStatus", [[Interface\Addons\weizPVP\Media\Textures\white16x16.tga]])

--> Bakgrounds
SM:Register("BACKGROUND", "weizPVP: #FFFFFF", [[Interface\Addons\weizPVP\Media\Textures\white16x16.tga]])
SM:Register("BACKGROUND", "weizPVP: Bar-BG", [[Interface\Addons\weizPVP\Media\Textures\bar-solid75.tga]])

--> Borders
SM:Register("BORDER", "weizPVP: Border", [[Interface\Addons\weizPVP\Media\Textures\white16x16.tga]])

--> Fonts
SM:Register("FONT", "Roboto Condensed Bold", [[Interface\Addons\weizPVP\Media\Fonts\RobotoCondensed-Bold.ttf]])
SM:Register(
    "FONT",
    "Roboto Condensed BoldItalic",
    [[Interface\Addons\weizPVP\Media\Fonts\RobotoCondensed-BoldItalic.ttf]]
)
SM:Register("FONT", "Accidental Presidency", [[Interface\Addons\weizPVP\Media\Fonts\Accidental-Presidency.ttf]])
SM:Register(
    "FONT",
    "Accidental Presidency Italic",
    [[Interface\Addons\weizPVP\Media\Fonts\Accidental-Presidency-Italic.ttf]]
)
SM:Register("FONT", "PT Sans Narrow Bold", [[Interface\Addons\weizPVP\Media\Fonts\PTSansNarrow-Bold.ttf]])

--> Sounds
SM:Register("SOUND", "weizPVP: MLG Air Horn 1", [[Interface\Addons\weizPVP\Media\Sounds\airhorn-1.ogg]])
SM:Register("SOUND", "weizPVP: Beep 1", [[Interface\Addons\weizPVP\Media\Sounds\beep-1.ogg]])
SM:Register("SOUND", "weizPVP: Beep 2", [[Interface\Addons\weizPVP\Media\Sounds\beep-2.ogg]])
SM:Register("SOUND", "weizPVP: Error 1", [[Interface\Addons\weizPVP\Media\Sounds\error-1.ogg]])
SM:Register("SOUND", "weizPVP: Notice 1", [[Interface\Addons\weizPVP\Media\Sounds\notice-1.ogg]])
SM:Register("SOUND", "weizPVP: Notice 2", [[Interface\Addons\weizPVP\Media\Sounds\notice-2.ogg]])
SM:Register("SOUND", "weizPVP: Notice 3", [[Interface\Addons\weizPVP\Media\Sounds\notice-3.ogg]])
SM:Register("SOUND", "weizPVP: Warning 1", [[Interface\Addons\weizPVP\Media\Sounds\warning-1.ogg]])
SM:Register("SOUND", "weizPVP: Warning 2", [[Interface\Addons\weizPVP\Media\Sounds\warning-2.ogg]])
SM:Register("SOUND", "weizPVP: Warning 3", [[Interface\Addons\weizPVP\Media\Sounds\warning-3.ogg]])
SM:Register("SOUND", "weizPVP: Warning 4", [[Interface\Addons\weizPVP\Media\Sounds\warning-4.ogg]])

-------------------------------------------------------------------------------
--|> DEFAULT OPTIONS
-------------------------------------------------------------------------------
weizPVP.defaults = {}
local defaults
do
    defaults = {
        profile = {
            Options = {
                DB_VERSION = "3.0",
                DEBUG = false,
                Addon = {
                    Enabled = true,
                    Visible = true,
                    EnabledInBattlegrounds = true,
                    EnabledInArena = false,
                    Halted = false,
                    VersionSplashSeen = false,
                    VersionNumber = 0,
                    Version = ""
                },
                Window = {
                    Locked = false,
                    Pinned = false,
                    Collapsed = false
                },
                --> LagMode
                LagMode = {
                    Enabled = false,
                    Active = false,
                    FPSThreshold = 30,
                    HaltEnabled = false,
                    HaltActive = false
                },
                --> Alerts
                AudioAlerts = {
                    DetectedPlayerSound = false,
                    DetectedPlayerSoundFile = "weizPVP: Notice 1"
                },
                --> Bars
                Bars = {
                    MaxNumBars = 20,
                    AlphaDead = 0.8,
                    AlphaInactive = 0.6,
                    AlphaDefault = 1,
                    RowHeight = 18,
                    VerticalSpacing = 1,
                    Texture = "weizPVP: StatusBar",
                    BarTexture = "weizPVP: Bar-BG",
                    BarSolid = "weizPVP: SolidStatus",
                    NameFont = "Accidental Presidency",
                    NameFontSize = 16,
                    LevelFont = "Accidental Presidency Italic",
                    LevelFontSize = 16,
                    GuildFont = "Accidental Presidency",
                    GuildFontSize = 14
                },
                --> Sorting
                Sorting = {
                    NearbyInactiveTimeout = 32,
                    NearbyActiveTimeout = 20
                },
                --> Frames
                Frames = {
                    BackgroundTexture = "weizPVP: #FFFFFF",
                    BorderTexture = "weizPVP: Border",
                    BackgroundColor = {
                        0.02,
                        0.02,
                        0.02,
                        0.5
                    },
                    Width = 251,
                    Height = 247,
                    X = 383,
                    Y = -4.5,
                    Scale = 1,
                    Point = "CENTER",
                    BorderSize = 4,
                    AutoResize = false,
                    --> Scroll Frame
                    Scroll = {
                        Height = 0
                    },
                    --> List Frame
                    List = {
                        Height = 0
                    },
                    --> Header Frame
                    Header = {
                        Height = 26,
                        BackgroundColor = {
                            0.01,
                            0.01,
                            0.01,
                            0.7
                        },
                        Font = "Roboto Condensed BoldItalic",
                        FontSize = 18,
                        X = 383,
                        Y = 132,
                        Scale = 1,
                        Width = 251,
                        FontOutline = "OUTLINE"
                    },
                    --> Footer Frame
                    Footer = {
                        Height = 18,
                        Font = "Accidental Presidency",
                        FontSize = 15,
                        FontOutline = "NONE",
                        BackgroundColor = {
                            0.01,
                            0.01,
                            0.01,
                            0.4
                        }
                    }
                },
                --> Stealth Alert
                StealthAlert = {
                    Enabled = true,
                    EnableSound = true,
                    SoundFile = "weizPVP: Warning 1"
                },
                --> Crosshair
                Crosshair = {
                    Enabled = true,
                    Alpha = 1,
                    OnlyOnKosTargets = false,
                    ShowBountyOverlay = false,
                    ColorOverride = false,
                    ColorOverrideRGBA = {
                        1,
                        0.2,
                        0.2,
                        0.6
                    },
                    NameEnabled = false,
                    GuildEnabled = false,
                    ExtraFX = false,
                    RotatingArrows = false,
                    LineThickness = 2,
                    Parallax = false
                }
            }
        }
    }

    --> Calculate additional defaults based on the dimentions of other properties
    --: Dynamic values adjusted from other values needed during initial frame construction
    -------------------------------------------------------------------------------
    defaults.profile.Options.Frames.List.Height =
        defaults.profile.Options.Bars.RowHeight + defaults.profile.Options.Bars.VerticalSpacing
    defaults.profile.Options.Frames.Scroll.Height =
        (defaults.profile.Options.Frames.List.Height * 12) + defaults.profile.Options.Bars.VerticalSpacing
    defaults.profile.Options.Frames.Height = defaults.profile.Options.Frames.Scroll.Height
    defaults.profile.Options.Frames.List.Height =
        defaults.profile.Options.Frames.List.Height * defaults.profile.Options.Bars.MaxNumBars

    weizPVP.defaults = defaults
end

-------------------------------------------------------------------------------
--|> LOAD DATABASE
--: Initializes static data tables
--: Loads saved variables or applies defaults if none exist
-------------------------------------------------------------------------------
function weizPVP:LoadDB()
    --> LOAD CHARACTER OPTIONS
    weizPVP.charDB = {}
    weizPVP.Options = {}
    weizPVP.charDB = LibStub("AceDB-3.0"):New("weizPVPOptions", defaults, true)
    weizPVP.Options = weizPVP.charDB.profile.Options

    --> LOAD GLOBAL PLAYER DB
    weizPVP.globalDB = {}
    weizPVP.PlayerDB = {}
    weizPVP.globalDB = LibStub("AceDB-3.0"):New("weizPVPDatabase", {}, false)
    weizPVP.PlayerDB = weizPVP.globalDB.global
end

-------------------------------------------------------------------------------
--> INIT CURRENT VALUES
-------------------------------------------------------------------------------
function weizPVP:BuildUIElements()
    --> Create Core Frames and Bars
    weizPVP:CreateCoreFrames()
    weizPVP:CreateBars()
    weizPVP:CreateStatusBar()
    if weizPVP.Options.Window.Collapsed == true then
        weizPVP.MinFrame()
    else
        weizPVP.MaxFrame()
    end
    weizPVP:CreateStealthAlertFrame()
    --> Setup Crosshair
    weizPVP:CreateCrosshair()
    if weizPVP.Options.Crosshair.Enabled then
        weizPVP:EnableCrosshair()
    else
        weizPVP:DisableCrosshair()
    end
end

-------------------------------------------------------------------------------
--|> LOCAL FUNCTIONS
-------------------------------------------------------------------------------
--> Fire Splash Alert if it hasnt been seen.
local function CheckSplash()
    weizPVP:CheckSplash() --> Redirecting to weizPVP GLOBAL. For Science.
end
local function UpdateOldFramePosition()
end

-------------------------------------------------------------------------------
--|> ON INITIALIZE
-------------------------------------------------------------------------------
function weizPVP:OnInitialize()
    weizPVP:LoadDB()
    weizPVP:BuildUIElements()

    --> Check for new version Splash Screen
    weizPVP:CreateSplashScreen()
    C_Timer_After(1, CheckSplash)
end

-------------------------------------------------------------------------------
--|> ON ENABLE
-------------------------------------------------------------------------------
function weizPVP:OnEnable()
    weizPVP.ManageBarsDisplayed()
    UpdateOldFramePosition()
    weizPVP.HeaderFrame:Show()
    weizPVP.CoreFrame:Show()
    weizPVP.StatusBar:Show()

    --> Enable Events
    weizPVP.EnableEvents()

    --> Pulse Timer
    weizPVP:ScheduleRepeatingTimer("PulseEvent", 2)

    --> Apply Saved Settings
    weizPVP.Addon.ENABLED = true

    weizPVP:SetCoreFramePosition()
end

-------------------------------------------------------------------------------
--|> ON DISABLE
-------------------------------------------------------------------------------
function weizPVP:OnDisable()
    --> Hide frames
    weizPVP.HeaderFrame:Hide()
    weizPVP.CoreFrame:Hide()
    weizPVP.StatusBar:Hide()

    --> Cancel Timers
    weizPVP:CancelAllTimers()

    --> Clear current player data
    weizPVP:ClearListData()

    --> Enable Events
    weizPVP.DisableEvents()

    --> Mark addon is disabled
    weizPVP.Addon.ENABLED = false
end

-------------------------------------------------------------------------------
--|> RESET ALL
-------------------------------------------------------------------------------
function weizPVP:ResetAll()
    if weizPVP.charDB then
        wipe(weizPVP.charDB)
    end

    if weizPVP.globalDB then
        wipe(weizPVP.globalDB)
    end

    if weizPVP.PlayerDB then
        wipe(weizPVP.PlayerDB)
    end

    if weizPVP.Options then
        wipe(weizPVP.Options)
    end

    wipe(weizPVP.charDB)
    wipe(weizPVP.globalDB)

    C_UI_Reload()
end

-------------------------------------------------------------------------------
--|> RESET OPTIONS
-------------------------------------------------------------------------------
function weizPVP:ResetOptions()
    if weizPVP.Options then
        wipe(weizPVP.Options)
    end

    if wipe(weizPVP.charDB) then
        wipe((weizPVP.charDB))
    end

    C_UI_Reload()
end
